@@ -1,7 +1,6 @@
 import requests, json, fnmatch, sys, os
 from metacat.util import to_str, to_bytes
 from metacat.auth import SignedToken, TokenLib
-from pythreader import Task, TaskQueue, Promise
 from urllib.parse import quote_plus, unquote_plus
 from metacat.auth import TokenAuthClientMixin
 
@@ -42,6 +41,9 @@ class WebAPIError(ServerError):
         #print("WebAPIError.json: body:", self.Body)
         return json.loads(self.Body)
         
+class NotFoundError(WebAPIError):
+    pass
+            
 class InvalidMetadataError(WebAPIError):
 
     def __str__(self):
@@ -84,7 +86,9 @@ class HTTPClient(object):
         response = requests.get(url, headers =headers)
         if response.status_code == INVALID_METADATA_ERROR_CODE:
             raise InvalidMetadataError(url, response.status_code, response.text)
-        if response.status_code != 200:
+        if response.status_code == 404:
+            raise NotFoundError(url, response.status_code, response.text)
+        elif response.status_code != 200:
             raise WebAPIError(url, response.status_code, response.text)
         return response.text
 
@@ -164,7 +168,8 @@ class MetaCatClient(HTTPClient, TokenAuthClientMixin):
         
         HTTPClient.__init__(self, server_url, token)
         self.AuthURL = auth_server_url or server_url + "/auth"
-        self.QueryQueue = TaskQueue(max_concurrent_queries)       
+        self.MaxConcurrent = max_concurrent_queries
+        self.QueryQueue = None
         
     def resfresh_token(self):
         if self.TokenFile:
@@ -209,13 +214,15 @@ class MetaCatClient(HTTPClient, TokenAuthClientMixin):
         Returns
         -------
         dict
-            dataset attributes
+            dataset attributes or None if the dataset was not found
         """        
         
         if namespace is not None:
             spec = namespace + ':' + name
-        item = self.get_json(f"data/dataset?dataset={spec}")
-        return item
+        try:
+            return self.get_json(f"data/dataset?dataset={spec}")
+        except NotFoundError:
+            return None
         
     def create_dataset(self, spec, frozen=False, monotonic=False, creator=None, metadata=None, metadata_requirements=None, description=""):
         """Creates new dataset. Requires client authentication.
@@ -531,7 +538,7 @@ class MetaCatClient(HTTPClient, TokenAuthClientMixin):
         Returns
         -------
         dict
-            dictionary with file information
+            dictionary with file information or None if the file was not found
 
             .. code-block:: python
 
@@ -551,18 +558,23 @@ class MetaCatClient(HTTPClient, TokenAuthClientMixin):
         -----
         Retrieving file provenance and metadata takes slightly longer time
         """        
-        assert (fid is None) != (name is None), 'Either name="namespace:name" or fid="fid" must be specified, but not both'
+        assert (fid is not None) or (did is not None) or (name is not None and namespace is not None), \
+            "Either DID or file id or namespace and name must be specified"
         with_meta = "yes" if with_metadata else "no"
         with_rels = "yes" if with_provenance else "no"
         url = f"data/file?with_metadata={with_meta}&with_provenance={with_rels}"
         if did:
-            namespace, name = parse_name(did)
+            namespace, name = parse_name(did, None)
+            assert namespace is not None, f"Invalid DID format: {did}"
         if name:
             url += f"&name={name}&namespace={namespace}"
         else:
             url += f"&fid={fid}"        
             
-        return self.get_json(url)
+        try: 
+            return self.get_json(url)
+        except NotFoundError:
+            return None
 
     def query(self, query, namespace=None, with_metadata=False, with_provenance=False, save_as=None, add_to=None):
         """Run file query. Requires client authentication if save_as or add_to are used.
@@ -602,7 +614,7 @@ class MetaCatClient(HTTPClient, TokenAuthClientMixin):
         results = self.post_json(url, query)
         return results
         
-    def async_query(self, query, data, **args):
+    def async_query(self, query, data=None, **args):
         """Run file query asynchronously. Requires client authentication if save_as or add_to are used.
         
         Parameters
@@ -621,34 +633,17 @@ class MetaCatClient(HTTPClient, TokenAuthClientMixin):
             argument to the ``async_query`` call. 
         
             See notes below for more on how to use this method.
-            
-        
-
         """
         
-        class QueryTask(Task):
-            def __init__(self, client, query, promise, args):
-                Task.__init__(self)
-                self.Client = client
-                self.Query = query
-                self.Args = args
-                self.Promise = promise
-                
-            def run(self):
-                #print("QueryTask: started:", self.Query)
-                try:    
-                    results = self.Client.query(self.Query, **self.Args)
-                except Exception:
-                    self.Promise.exception(*sys.exc_info())
-                else:
-                    self.Promise.complete(results)
-                    
-        p = Promise(data)
-        t = QueryTask(self, query, p, args)
-        self.QueryQueue << t
-        #print ("Task added")
-        return p
-        
+        if self.QueryQueue is None:
+            try:    
+                from pythreader import TaskQueue
+            except ModuleNotFoundError:
+                raise ModuleNotFoundError("pythreader module required for asynchronous queries. Use: pip install 'pythreader>=2.7.0'")
+            self.QueryQueue = TaskQueue(self.MaxConcurrent)
+            
+        return self.QueryQueue.add_lambda(self.query, query, promise_data=data, **args)
+
     def wait_queries(self):
         self.QueryQueue.waitUntilEmpty()
     
@@ -690,10 +685,13 @@ class MetaCatClient(HTTPClient, TokenAuthClientMixin):
         Returns
         -------
         dict 
-            Namespace information
+            Namespace information or None if the namespace was not found
         """
         
-        return self.get_json(f"data/namespace?name={name}")
+        try: 
+            return self.get_json(f"data/namespace?name={name}")
+        except NotFoundError:
+            return None
         
     def get_namespaces(self, names):
         """Creates new namespace
